(* This program is free software; you can redistribute it and/or modify it under  *)
(* the terms of the GNU General Public License as published by the Free Software  *)
(* Foundation; either version 2 of the License, or (at your option) any later     *)
(* version.                                                                       *)
(*                                                                                *)  
(* This program is distributed in the hope that it will be useful, but WITHOUT    *)
(* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  *)
(* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. *)
(*                                                                                *)
(* Programmed by Alok Menghrajani.                                                *)
(* http://www.alokonline.com/projets/compiler/simpleb.html                        *)
(*                                                                                *)
(* Description: this file is used by ocamllex to create lexer.ml, which contains  *)
(*              information used by Lexing to create a lexical buffer. The        *)
(*              lexical buffer produces tokens defined in grammar.mly that are    *)
(*              then used to build an abstract syntax tree. If there is an error  *)
(*              (i.e. if the lexer encounters a character that doesn't match one  *)
(*              of the rules) Errormsg.error is called.                           *)
(*                                                                                *)
(* X-reference: absyn.ml, lexer.mll, symbol.ml, simpleb.ml (the main program)     *)


{
 open Grammar
 exception Error of string

 let debug = ref false

 let printf s = if !debug then Printf.printf s else ()

 let lstart = Lexing.lexeme_start
 let lend = Lexing.lexeme_end
 let get = Lexing.lexeme
     
 let bump_line n = Errormsg.startNewline n

 let commentLevel = ref 0
 let commentPos = ref 0
 let linCom = ref false

 let lineNum = ref 1
 let linePos = ref [1]

 let enterComment yypos =
   commentLevel := !commentLevel + 1;
   commentPos := yypos
    
 let linComStart yypos = 
   linCom := true;
   commentPos := yypos

 let isLinCom () = !linCom
 let linComEnd () = linCom := false

 let exitComment () =
   commentLevel := !commentLevel - 1;
   !commentLevel = 0

 let isCom () =
   !commentLevel > 0

 let eof () =
   if (!commentLevel > 0) then
     (Errormsg.error (!commentPos,!commentPos) "Unterminated comment")
   else
     ();
   EOF
       
} 

let alpha       = ['a'-'z' 'A'-'Z']
let digit       = ['0'-'9']
let id          = (alpha|'_')(digit|alpha|'_')*
let nl          = ('\r' | '\n' | "\r\n")
 
 rule token = 
    parse
   eof                     { eof() }
|  [' ''\t']+              { token lexbuf }

|  nl                      {
  lineNum := !lineNum + 1;
  linePos := (lstart lexbuf) :: !linePos;
  bump_line (lstart lexbuf);
  if isCom() then (
    printf "========== C\n";
    token lexbuf
   )  else (
    printf "==========\n";
    token lexbuf ) }

|  "++"	                   { PLUSPLUS }
|  "--"                    { MINUSMINUS }
|  ','                     { COMMA }
|  '!'                     { NOT }
|  "&&"                    { AND }
|  "||"                    { OR }
|  '^'  		   { XOR }
|  "=="                    { EQ }
|  "!="                    { NE }
|  '<'                     { LT }
|  "<="                    { LE }
|  '>'                     { GT }
|  ">="                    { GE }
|  "break"                 { BREAK }
|  "continue"		   { CONTINUE }
|  "else"                  { ELSE }
|  "false"                 { FALSE }
|  "for"                   { FOR }
|  "if"                    { IF }
|  "int"                   { INT }
|  "true"                  { TRUE }
|  "while"                 { WHILE }
|  "writeint"              { WRITEINT }
|  '{'                     { LBRACE }
|  '}'                     { RBRACE }
|  '('                     { LPAREN }
|  ')'                     { RPAREN }
|  '+'			   { PLUS }
|  '*'			   { TIMES }
|  '-'			   { MINUS }
|  '/'		           { DIVIDE }
|  '='		           { ASSIGN }
|  '%'		           { MOD }
|  "+="			   { PLUSASSIGN }
|  "*="			   { TIMESASSIGN }
|  "-="			   { MINUSASSIGN }
|  "/="		           { DIVIDEASSIGN }
|  "%="		           { MODASSIGN }
|  '~'		           { NEG }
|  ';'		           { SEMICOL }
|  "//"                    { linComStart (lstart lexbuf);
			     comment lexbuf }
|  "/*"                    { printf "/*\n"; enterComment (lstart lexbuf);
			     comment lexbuf }
|  "*/"                    { Errormsg.error (lstart lexbuf,
						lend lexbuf - 1)
			       "Unbalanced comment"; token lexbuf }
|  digit+                  { INTCONST(Int32.of_string(get lexbuf)) }
|  id                      { ID(Symbol.symbol (get lexbuf)) }
|  _                       { Errormsg.error (lstart lexbuf,
						lend lexbuf - 1)
			       ("illegal character '"
				^ get lexbuf
				^ "'"); token lexbuf }


and comment =
  parse "/*"
    {
     if not (isLinCom ()) then (
       printf "/*isnotlinecom\n";
       enterComment (lstart lexbuf)
     ) else (
       printf "/*islinecom\n";
       () );
     comment lexbuf }
    
|  "*/"
    {
     if not (isLinCom ()) && exitComment () then (
       printf "*/isnotlinecom\n";
       token lexbuf
      ) else (
       printf "*/islinecom or stillincom\n";
       comment lexbuf )
   }
    
|  nl
    {
     if isLinCom () then (
       printf "\\nislinecom\n";
       linComEnd ()
      ) else (
       printf "\\nisnotlinecom\n";
       ());
     lineNum := !lineNum + 1;
     linePos := (lstart lexbuf) :: !linePos;
     bump_line (lstart lexbuf);
     if isCom() then (
       printf "========== C\n";
       comment lexbuf
      )  else (
       printf "==========\n";
       token lexbuf )}
    
|  eof
    { eof() }
|  _
    { comment lexbuf }
    
